﻿'***************************************************************
' Module    : c3600.vb
' Function  : VB.NET application include for CPS 3600 API
' Copyright : (C) 2016 CPS
'***************************************************************

Imports System.Runtime.InteropServices
Module c3600

    ' Constants
    Public Const MAX_SYSTEMS As Int32 = 4               ' Maximum number of power supply systems.
    Public Const MAX_DEVICES As Int32 = 16              ' Maximum number of devices in one system.
    Public Const MAX_PORTS As Int32 = 8                 ' Maximum number of COM ports.
    Public Const INFINITE_WAIT As Int32 = &HFFFFFFFF    ' Unlimited wait time.

    ' Error codes returned by API functions
    Public Const C3600_ERR_OK As Int32 = 0
    Public Const C3600_ERR_INVALID_PORT As Int32 = (-1)       ' Incorrect COM port number.
    Public Const C3600_ERR_INVALID_SYSID As Int32 = (-2)      ' Invalid system ID.
    Public Const C3600_ERR_INVALID_DEVID As Int32 = (-3)      ' Invalid device ID.
    Public Const C3600_ERR_INVALID_VALUE As Int32 = (-4)      ' Invalid parameter value.
    Public Const C3600_ERR_WRITE_FAIL As Int32 = (-5)         ' Cannot write to port.
    Public Const C3600_ERR_READ_FAIL As Int32 = (-6)          ' Cannot read from port.
    Public Const C3600_ERR_READ_SUM As Int32 = (-7)           ' Checksum error in received packet.
    Public Const C3600_ERR_READ_LEN As Int32 = (-8)           ' Length error in received packet.
    Public Const C3600_ERR_READ_TIMEOUT As Int32 = (-9)       ' Timed out waiting for received packet byte.
    Public Const C3600_ERR_ALREADY_OPEN As Int32 = (-10)      ' Attempted to open a system that is already open.
    Public Const C3600_ERR_CLOSED As Int32 = (-11)            ' System is closed.
    Public Const C3600_ERR_CREATE_LOCK As Int32 = (-12)       ' Can't create mutex.
    Public Const C3600_ERR_LOCK_TIMEOUT As Int32 = (-13)      ' Timed out waiting for device lock.
    Public Const C3600_ERR_DEV_UNAVAILABLE As Int32 = (-14)   ' Device is unresponsive or not present.
    Public Const C3600_ERR_FUNC_UNSUPPORTED As Int32 = (-15)  ' Function unavailable on this device.
    Public Const C3600_ERR_METER_OVERRANGE As Int32 = (-16)   ' Meter ADC input outside normal range.

    Public Const C3600_ERR_LOCK_ABANDONED As Int32 = (-17)     ' The specified object is a mutex object that was not released by the thread that owned the mutex object before the owning thread terminated. Ownership of the mutex object is granted to the calling thread, and the mutex is set to nonsignaled. If the mutex was protecting persistent state information, you should check it for consistency.
    Public Const C3600_ERR_INVALID_HANDLE As Int32 = (-19)     ' CreateFile() returned INVALID_HANDLE_VALUE
    Public Const C3600_ERR_GETCOMMMODEMSTATUS As Int32 = (-20) ' Failed to read DTR/RTS modem control lines
    Public Const C3600_ERR_SETCOMMSTATE As Int32 = (-21)       ' Failed to set DTR/RTS modem control lines

    Public Const C3600_ERR_INTERNAL As Int32 = (-99)          ' System error (use GetLastError() or equivalent to determine system error code).

    ' Flags returned by C3600_GetFaultFlags()
    Public Const FAULT_VMETER_ADC As Int32 = &H40       ' Voltage metering ADC malfunction.
    Public Const FAULT_IMETER_ADC As Int32 = &H20       ' Current metering ADC malfunction.
    Public Const FAULT_SETPOINT_DAC As Int32 = &H8      ' Setpoint DAC malfunction.

    ' Flags returned by C3600_GetStatusFlags()
    Public Const STATUS_RESET As Int32 = &H80           ' Device has been reset.
    Public Const STATUS_OVER_TEMP As Int32 = &H20       ' Overtemperature input is asserted.
    Public Const STATUS_FAULT As Int32 = &H10           ' One or more faults detected; call C3600_GetFaultFlags() for details.
    Public Const STATUS_EEBUSY As Int32 = &H8           ' EEPROM write operation in progress.
    Public Const STATUS_OUTPUT_ENABLED As Int32 = &H4   ' Power supply output is enabled.
    Public Const STATUS_OVER_CURRENT As Int32 = &H2     ' Overcurrent input is asserted.
    Public Const STATUS_OVER_VOLTAGE As Int32 = &H1     ' Overvoltage input is asserted.

    ' Meter types
    Public Const METER_V As Int32 = 0       ' Volts
    Public Const METER_I As Int32 = 1       ' Amps
    Public Const METER_T As Int32 = 2       ' Degrees C

    ' Public types ------------------------------------------

    Enum DEVTYPE            ' DEVICE OUTPUT TYPES
        NONE = 0            '  Device unavailable (device detected, but valid stored config indicates it is not used).
        UNKNOWN = 1         '  Unknown device type (device detected, but stored config is invalid).
        ACCELERATOR = 2     '  Accelerator
        BIAS = 3            '  Bias
        EXTRACTOR = 4       '  Extractor
        FILAMENT = 5        '  Filament
        LENS = 6            '  Lens
        SUPPRESSOR = 7      '  Suppressor
    End Enum
    Public Const NUM_DEVTYPES = 8

    Enum SIGTYPE As UInt32                  ' SETPOINT TYPE
        VOLTS = 0
        AMPS = 1
    End Enum

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Structure OUT_RANGE                     ' DEVICE OUTPUT RANGE
        Public max As Double                '  Maximum output.
        Public min As Double                '  Minimum output.
    End Structure

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Structure DEV_INFO                      ' DEVICE CONSTANTS
        <MarshalAs(UnmanagedType.ByValArray, sizeconst:=2)> Public range() As OUT_RANGE '  Voltage/current output ranges in volts/amps.
        Public sptype As SIGTYPE            '  Setpoint type: VOLTS | AMPS.
        Public is_detected As Int32         '  Device was detected during system open.
        Public is_valid As Int32            '  Device configuration data has valid checksum.
        Public fwversion As UInt16          '  Firmware version number.
    End Structure

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Structure SYS_INFO                      ' SYSTEM CONSTANTS
        Public is_valid As Int32                                                                    '  System configuration data has valid checksum.
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=8)> Public model As String                   '  Model number.
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=13)> Public serial_num As String             '  Serial number.
    End Structure

    ' API functions ---------------------------------------

    Private Const DLL As String = "c3600.dll"

    <DllImport(DLL)> _
        Public Function C3600_ApiOpen(ByRef version As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_ApiClose() As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_OpenSystem(ByVal sysid As Int32, ByVal port As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_CloseSystem(ByVal sysid As Int32) As Int32
    End Function

    <DllImport(DLL)> _
         Public Function C3600_GetSysInfo(ByVal sysid As Int32, ByRef sysinfo As SYS_INFO, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetDevInfo(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByRef devinfo As DEV_INFO, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetFaultFlags(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByRef flags As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetStatusFlags(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByRef flags As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetMeter(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByVal meterid As Int32, ByRef data As Double, ByRef rawdata As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetOutput(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByRef data As Double, ByRef rawdata As Int32, ByRef enable As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SetOutput(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByVal data As Double, ByVal enable As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SetWatchdog(ByVal sysid As Int32, ByVal msec As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SysReset(ByVal sysid As Int32, ByVal hard As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetAlarm(ByVal sysid As Int32, ByRef alarm As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetErrorString(ByVal errcode As Int32) As String
    End Function



    '*********************************************************************
    ' Module    : c3600_proprietary.vb
    ' Function  : CPS proprietary application include for CPS 3600 API
    ' Target OS : OS-independent
    ' Copyright : (C) 2016 CPS
    '*********************************************************************

    Enum DEVCLASS              ' DEVICE ARCHITECTURE CLASSIFICATIION
        DEV0 = 0               '   Slave device 0 (primary device on a "slave" unit).
        DEV1 = 1               '   Slave device 1 (secondary device on a "slave" unit).
        SAD = 2                '   Master unit's stand-alone device.
        DEVX = 3               '   Unimplemented device class.
    End Enum

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
        Structure CAL_CONST             ' Calibration constants for a data converter's min or max raw value: corrval = baseval + tslope * (DegC - toffset)
        Public baseval As Int32         '   Ideal (uncorrected) raw value corresponding to device min/max output level at ref temp.
        Public toffset As Int32         '   Temperature correction -- offset.
        Public tslope As Single         '   Temperature correction -- slope.
    End Structure

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Structure CAL_PAIR                  ' Calibration for one data converter (adc or dac):
        Public max As CAL_CONST         '   Maximum output level.
        Public min As CAL_CONST         '   Minimum output level.
    End Structure

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Structure DEV_CONFIG
        Public setpoint As CAL_PAIR     ' Setpoint conversion values.
        <MarshalAs(UnmanagedType.ByValArray, sizeconst:=2)> Public meter() As CAL_PAIR  ' Conversion values for voltage/current meters.
        <MarshalAs(UnmanagedType.ByValArray, sizeconst:=2)> Public range() As OUT_RANGE ' Output voltage/current ranges in volts/amps.
        Public setpoint_type As SIGTYPE ' Enumerated setpoint type (VOLTS | AMPS).
        Public devtype As DEVTYPE       ' Power supply type.
        ' Read only:
        Public devclass As DEVCLASS     ' Device class.
        Public devaddr As Int32         ' Device address: (cluster/2, unit/2, devnum).
        Public type_index As Int32      ' Index in system's list of devices of this type.
        Public fwversion As UInt16      ' Firmware version number.
    End Structure

    'typedef struct SYS_CONFIG {                  // System personality:
    '  int             num_clusters;              //   Number of clusters in system.
    '  char            model[MODEL_BUFLEN==8];    //   Power supply model number string.
    '  char            serial_num[SN_BUFLEN==13]; //   Serial number string.
    '} SYS_CONFIG;

    <StructLayout(LayoutKind.Sequential, Pack:=1)> _
    Structure SYS_CONFIG
        Public num_clusters As Int32    ' Number of clusters in system.
        'Public model As UInt16          ' System model number.
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=8)> Public model As String                   '  Model number.
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=13)> Public serial_num As String             '  Serial number.
    End Structure

    ' API functions for factory use only ---------------------------------------

    <DllImport(DLL)> _
        Public Function C3600_ReadPort(ByVal sysid As Int32, ByVal devaddr As Int32, ByRef buf As Byte, ByRef nbytes As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_WritePort(ByVal sysid As Int32, ByVal devaddr As Int32, ByRef buf As Byte, ByRef nbytes As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetEEPROM(ByVal sysid As Int32, ByVal devaddr As Int32, ByVal eeaddr As Int32, ByVal data As Byte, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SetEEPROM(ByVal sysid As Int32, ByVal devaddr As Int32, ByVal eeaddr As Int32, ByVal value As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SetDevConfig(ByVal sysid As Int32, ByVal devnum As Int32, ByRef cfg As DEV_CONFIG, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetDevConfig(ByVal sysid As Int32, ByVal devnum As Int32, ByRef cfg As DEV_CONFIG, ByRef is_valid As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SetSysConfig(ByVal sysid As Int32, ByRef cfg As SYS_CONFIG, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_GetSysConfig(ByVal sysid As Int32, ByRef cfg As SYS_CONFIG, ByRef is_valid As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
        Public Function C3600_SetOutputRaw(ByVal sysid As Int32, ByVal devtyp As DEVTYPE, ByVal devindex As Int32, ByVal raw As Integer, ByRef data As Double, ByVal enable As Int32, ByVal maxtime As Int32) As Int32
    End Function

    <DllImport(DLL)> _
    Public Function C3600_HalGetLastError() As Int32
    End Function

End Module
